package net.locusworks.lsproject.gui;

import java.awt.BorderLayout;
import java.awt.event.*;
import java.util.ArrayList;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import net.locusworks.lsproject.driver.Program;
import net.locusworks.lsproject.object.Network;
import net.locusworks.lsproject.object.Router;
import net.locusworks.lsproject.object.helper.DijkstraCallback;

/**
 * 
 */
public class RoutingTableWindow extends JFrame implements WindowListener {
	private static final long serialVersionUID = 1768859322938299953L;
	
	private Router router 	= null;
	private JTable table 	= null;
	private RoutingTableModel tableModel = new RoutingTableModel();
	private String set		= "";
	
	/**
	 * Display a new routing table window.
	 * @param _router
	 */
	public RoutingTableWindow(Router _router) {
		super("Routing table - " + _router.getIpAddress());
		
		router = _router;
		
		initializeComponents();
		
		setVisible(true);
	}
	
	/**
	 * Initialize the GUI components used by this window.
	 */
	private void initializeComponents() {
		setLayout(new BorderLayout());
		
		table = new JTable(tableModel);
		JScrollPane scrollPane = new JScrollPane(table);
		
		add(table.getTableHeader(), BorderLayout.PAGE_START);
		add(scrollPane, BorderLayout.CENTER);
		
		setSize(425, 200);
		
		router.getRoutingTable().registerCallback(tableModel);
		
		router.getRoutingTable().execute(router, null);
	}
	
	/**
	 * Allow access to which router this window is displaying.
	 * @return Router object.
	 */
	public Router getRouter() {
		return router;
	}
	
	/**
	 * Model used for displaying the routing table.
	 */
	@SuppressWarnings("serial")
	private class RoutingTableModel extends AbstractTableModel implements DijkstraCallback {
		private ArrayList<String> columnNames = new ArrayList<String>();
		private Object[][] data;
		private int currentRow = 0;
		
		public RoutingTableModel() {
		}
		
		/*
		 * Handle callback.
		 */
		
		@Override public void reset() {
			set = router.getIpAddress();
			columnNames.clear();
			currentRow = 0;
			
			columnNames.add("N'");
			columnNames.add(router.toString());
			for (Router r : Network.getInstance().getRouters()) {
				if (r != router) {
					columnNames.add(r.getIpAddress());
				}
			}
			
			fireTableStructureChanged();
			
			data = new Object[Network.getInstance().getRouters().size()][columnNames.size()];
			
			for (int i = 0; i < data.length; i++) {
				for (int j =0; j < data[i].length; j++) {
					data[i][j] = "";
					fireTableCellUpdated(i, j);
				}
			}
			
			setValueAt(set, 0, 0);
			
			fireTableDataChanged();
			
			System.out.println("Routing table " + router + " reset.");
		}
		
		@Override public void predecessorAdded(Router key, Router value) {
			if (key == router) return;
			final int columnIndex = getColumnByName(value.getIpAddress());
			
			setValueAt(key.getIpAddress() + "," + getValueAt(currentRow, columnIndex), currentRow, columnIndex);
		}

		@Override public void settledRouterAdded(Router r) {
			if (set.contains(r.getIpAddress())) return;
			
			currentRow++;
			
			set += (set.length() > 0 ? "," : "") + r.getIpAddress();
			
			final int rowIndex = set.split("\\,").length - 1;

			setValueAt(set, rowIndex, 0);
		}

		@Override public void shortestDistanceUpdated(Router router, int cost) {
			final int column = getColumnByName(router.getIpAddress());
			
			setValueAt(cost, currentRow, column);
		}
		
		/*
		 * Override AbstractTableModel functions.
		 */
		
		@Override public String getColumnName(int columnNumber) {
			return columnNames.get(columnNumber);
		}
		
		public int getColumnByName(String columnName) {
			for (int i = 0; i < columnNames.size(); i++) {
				if (columnNames.get(i).equals(columnName)) {
					return i;
				}
			}
			
			return -1;
		}
		
		public int getRowByFirstCellName(String cellName) {
			for (int i = 1; i < getRowCount(); i++) {
				if (columnNames.get(i).endsWith(cellName)) {
					return i;
				}
			}
			
			return -1;
		}
		
		@Override public int getColumnCount() {
			return columnNames.size();
		}

		@Override public int getRowCount() {
			return data.length;
		}

		@Override public Object getValueAt(int rowIndex, int columnIndex) {
			return data[rowIndex][columnIndex];
		}
		
		@Override public void setValueAt(Object value, int rowIndex, int columnIndex) {
			data[rowIndex][columnIndex] = value;
			
			super.fireTableCellUpdated(rowIndex, columnIndex);
		}
	}

	@Override public void windowClosing(WindowEvent e) {
		router.getRoutingTable().unregisterCallback(tableModel);
	}
	
	@Override public void windowActivated(WindowEvent arg0) {}
	@Override public void windowClosed(WindowEvent arg0) {}
	@Override public void windowDeactivated(WindowEvent arg0) {}
	@Override public void windowDeiconified(WindowEvent arg0) {}
	@Override public void windowIconified(WindowEvent arg0) {}
	@Override public void windowOpened(WindowEvent arg0) {}
}
