package net.locusworks.lsproject.object;

import java.io.Serializable;

/**
 * A Connection represents a single wire (i.e. the connection between two routers).
 */
public class Connection implements Serializable {
	private static final long serialVersionUID = -1883775251010127340L;
	
	private int cost;
	private String ipAddress1;
	private String ipAddress2;
	
	/**
	 * Construct a Connection object.
	 * @param _ipAddress1 Address of one router on the connection.
	 * @param _ipAddress2 Address of the other router on the connection.
	 * @param _cost Cost of the wire.
	 */
	public Connection(String _ipAddress1, String _ipAddress2, int _cost) {
		ipAddress1 = _ipAddress1;
		ipAddress2 = _ipAddress2;
		cost = _cost;
	}
	
	/**
	 * Synchronized (thread-safe) method for obtaining the cost of this connection.
	 * @return Cost (type integer) of the wire.
	 */
	public synchronized int getCost() {
		return cost;
	}
	
	/**
	 * Synchronized (thread-safe) method for setting a new cost for this connection.
	 * @param _cost Cost (type integer) of the wire.
	 */
	public synchronized void setCost(int _cost) {
		cost = _cost;
	}
	
	/**
	 * A single connection has at most two participants.  This retrieves IP Address 1
	 * (not ordered in any way; this is simply the first parameter of the Connection constructor).
	 * @return IP address of participant #1 of this connection.
	 */
	public String getFirstParticipantAddress() {
		return ipAddress1;
	}
	
	/**
	 * A single connection has at most two participants.  This retrieves IP Address 2
	 * (not ordered in any way; this is simply the second parameter of the Connection constructor).
	 * @return IP address of participant #2 of this connection.
	 */
	public String getSecondParticipantAddress() {
		return ipAddress2;
	}
	
	/**
	 * If the participants of this connections ever need to be modified, they can be done so through
	 * this method. 
	 * @param _ipAddress1 IP Address of the first participant.
	 * @param _ipAddress2 IP Address of the second participant.
	 */
	public void setParticipants(String _ipAddress1, String _ipAddress2) {
		ipAddress1 = _ipAddress1;
		ipAddress2 = _ipAddress2;
	}
}
